#include "ShaderConstants.fxh"
#include "util.fxh"

struct PS_Input
{
	float4 position : SV_Position;
	float3 worldpos : WORLDPOS;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

float3 saturation(float3 rgb, float adjust)
   { float3 intensity = dot(rgb, float3(.2125, .7154, .0721)); //} 
     return lerp(intensity, rgb, adjust);
 //} saturation function
  }

  bool red_sampler(float3 x, float3 y){
if(((all(x < float3(.85,.25,.25)) && all(x > float3(.825,.1,.1))) &&! (all(y > float3(.0,.0,.0)) || all(y < float3(.5,.5,.5))))){
return true;}else{return false;}}

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif

 float2 shiftclr = float2(.019, .0);
 float2 shiftpixel = float2(.03125, .0);

// detection materials
#if USE_TEXEL_AA
  float3 clrsampler = texture2D_AA(TEXTURE_0,TextureSampler0,PSInput.uv0+shiftclr).rgb;
  float3 clrpixel = texture2D_AA(TEXTURE_0,TextureSampler0,PSInput.uv0+shiftpixel).rgb;
#else
  float3 clrsampler = TEXTURE_0.Sample(TextureSampler0,PSInput.uv0+shiftclr).rgb;
  float3 clrpixel = TEXTURE_0.Sample(TextureSampler0,PSInput.uv0+shiftpixel).rgb;
#endif

float3 torchColor = float3(1.5, 0.6, 0.2);
float3 torchOut = torchColor * min(max(dot(PSInput.uv1.x - 0.2, 0.5), 0.0), max(dot(PSInput.uv1.x, 0.6), 0.0) * 0.4);
diffuse.rgb += torchOut;

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
if(!(red_sampler(clrsampler, clrpixel))){
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
}
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

float3 N = normalize(cross(ddx(PSInput.worldpos), ddy(PSInput.worldpos)));

if(PSInput.color.b > PSInput.color.r){
diffuse.a -=.3;
}

diffuse.rgb = saturation(diffuse.rgb, 1.37);
diffuse.rgb *= lerp(1.4, lerp(1., .8, PSInput.uv1.y), max(max(0., -N.y), abs(N.x)));
diffuse.rgb = lerp(diffuse.rgb, lerp(FOG_COLOR.rgb, diffuse.rgb, .7), clamp(length(PSInput.worldpos) / 70., .0, 1.));

#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}